@@ -9,8 +9,8 @@ use pyo3::{
 use rr_util::{
     lru_cache::TensorCacheRrfs,
     py_types::{scalar_to_tensor, Tensor, PY_UTILS},
-    rrfs::{save_tensor_rrfs, tensor_from_hash, tensor_from_hash_prefix},
     sv,
+    tensor_db::{get_tensor_prefix, save_tensor},
     tensor_util::{TorchDeviceDtype, TorchDeviceDtypeOp},
 };
 use rustc_hash::FxHashMap as HashMap;
@@ -115,7 +115,7 @@ impl ArrayConstant {
                 .get_tensor(hash_base16.to_owned())
                 .map(|value| ArrayConstant::new(value, name));
         }
-        tensor_from_hash_prefix(hash_base16).map(|value| ArrayConstant::new(value, name))
+        get_tensor_prefix(hash_base16).map(|value| ArrayConstant::new(value, name))
     }
 
     pub fn randn(shape: Shape) -> Self {
@@ -195,7 +195,7 @@ impl ArrayConstant {
     }
 
     pub fn save_rrfs(&self) -> Result<String> {
-        save_tensor_rrfs(self.value.clone())
+        save_tensor(self.value.clone(), false).map(|_| self.tensor_hash_base16())
     }
 
     pub fn tensor_hash_base16(&self) -> String {
@@ -204,7 +204,7 @@ impl ArrayConstant {
 
     #[staticmethod]
     pub fn from_hash(name: Option<String>, hash_base16: &str) -> Result<Self> {
-        tensor_from_hash(hash_base16).map(|value| ArrayConstant::new(value, name))
+        get_tensor_prefix(hash_base16).map(|value| ArrayConstant::new(value, name))
     }
 
     #[staticmethod]
