@@ -330,7 +330,7 @@ pub trait CircuitNode: CircuitNodeInit {
     where
         Self: Clone,
     {
-        repr_circuit_deep_compiler(&self.clone().c(), false, false)
+        repr_circuit_deep_compiler(&self.clone().c(), false, false, false)
     }
 
     fn compiler_print(&self)
@@ -1250,6 +1250,9 @@ pub enum ConstructError {
     #[error("ModuleNode got unknown keyword argument, {argument}")]
     ModuleNodeUnknownArgument { argument: String },
 
+    #[error("ModuleNode extract: not all leaves present in circuit, {subcirc:?}")]
+    ModuleNodeExtractNotPresent { subcirc: CircuitRc },
+
     #[error("Named axis higher than rank")]
     NamedAxisAboveRank {},
 
@@ -1731,7 +1734,7 @@ impl PyCircuitBase {
     }
 
     fn __repr__(&self) -> String {
-        self.compiler_repr(true, true)
+        self.compiler_repr(true, true, false)
     }
 
     #[getter]
@@ -1764,14 +1767,17 @@ impl PyCircuitBase {
         print_circuit_stats(&self.0)
     }
 
-    #[args(bijection = "true", shape_only_necessary = "false")]
-    fn compiler_repr(&self, bijection: bool, shape_only_necessary: bool) -> String {
-        repr_circuit_deep_compiler(self, bijection, shape_only_necessary)
+    #[args(bijection = "false", shape_only_necessary = "true", arrows = "false")]
+    fn compiler_repr(&self, bijection: bool, shape_only_necessary: bool, arrows: bool) -> String {
+        repr_circuit_deep_compiler(self, bijection, shape_only_necessary, arrows)
     }
 
-    #[args(bijection = "true", shape_only_necessary = "false")]
-    fn compiler_print(&self, bijection: bool, shape_only_necessary: bool) {
-        rr_util::python_println!("{}", self.compiler_repr(bijection, shape_only_necessary))
+    #[args(bijection = "false", shape_only_necessary = "true", arrows = "false")]
+    fn compiler_print(&self, bijection: bool, shape_only_necessary: bool, arrows: bool) {
+        rr_util::python_println!(
+            "{}",
+            self.compiler_repr(bijection, shape_only_necessary, arrows)
+        )
     }
 
     fn numel(&self) -> BigUint {
