@@ -5,7 +5,10 @@ use macro_rules_attribute::apply;
 use once_cell::sync::Lazy;
 use pyo3::{prelude::*, types::PyDict};
 use regex::Regex;
-use rr_util::tensor_util::{Shape, TorchDeviceDtype};
+use rr_util::{
+    pycall,
+    tensor_util::{Shape, TorchDeviceDtype},
+};
 use rustc_hash::FxHashMap as HashMap;
 use uuid::{uuid, Uuid};
 
@@ -121,35 +124,45 @@ impl ModuleNodeSpec {
         Ok(result)
     }
 
-    pub fn expand(&self, nodes: &Vec<CircuitRc>) -> Result<CircuitRc> {
-        let key: (ModuleNodeSpec, Vec<HashBytes>) =
-            (self.clone(), nodes.iter().map(|x| x.info().hash).collect());
+    pub fn expand(&self, nodes: &Vec<CircuitRc>, name_prefix: Option<String>) -> Result<CircuitRc> {
+        let key: (ModuleNodeSpec, Vec<HashBytes>, Option<String>) = (
+            self.clone(),
+            nodes.iter().map(|x| x.info().hash).collect(),
+            name_prefix.clone(),
+        );
 
         if let Some(result) = MODULE_EXPANSIONS.with(|cache| cache.borrow().get(&key).cloned()) {
             return Ok(result);
         }
-
         let shapes = nodes.iter().map(|x| x.info().shape.clone()).collect();
-        let expanded_shape = self.expand_shape(&shapes)?;
-        let result = replace_nodes(
-            expanded_shape,
-            &nodes
-                .iter()
-                .enumerate()
-                .map(|(i, n)| {
-                    (
-                        Symbol::new(
-                            n.info().shape.clone(),
-                            ModuleNodeSpec::EXPAND_PLACEHOLDER_UUID,
-                            Some(format!("{}_{:?}", i, n.info().shape)),
-                        )
-                        .info()
-                        .hash,
-                        n.clone(),
+        let mut expanded_shape = self.expand_shape(&shapes)?;
+        println!("expanded shape");
+        expanded_shape.compiler_print();
+        let node_mapping: HashMap<HashBytes, CircuitRc> = nodes
+            .iter()
+            .enumerate()
+            .map(|(i, n)| {
+                (
+                    Symbol::new(
+                        n.info().shape.clone(),
+                        ModuleNodeSpec::EXPAND_PLACEHOLDER_UUID,
+                        Some(format!("{}_{:?}", i, n.info().shape)),
                     )
-                })
-                .collect(),
-        );
+                    .info()
+                    .hash,
+                    n.clone(),
+                )
+            })
+            .collect();
+        if let Some(prefix) = &name_prefix {
+            expanded_shape = deep_map_unwrap(expanded_shape, |x| {
+                if let Some(n) = x.name()  && !node_mapping.contains_key(&x.info().hash){
+                    return x.clone().rename(Some(prefix.clone() + n));
+                }
+                x.clone()
+            })
+        }
+        let result = replace_nodes(expanded_shape, &node_mapping);
         MODULE_EXPANSIONS.with(|cache| cache.borrow_mut().insert(key, result.clone()));
         Ok(result)
     }
@@ -163,6 +176,26 @@ impl ModuleNodeSpec {
         }
         hasher.finalize().into()
     }
+    pub fn map_circuit<F>(&self, f: F, name: Option<Option<String>>) -> Result<Self>
+    where
+        F: Fn(CircuitRc) -> Result<CircuitRc>,
+    {
+        Ok(ModuleNodeSpec {
+            input_specs: self.input_specs.clone(),
+            name: name.unwrap_or(self.name.clone()),
+            spec_circuit: f(self.spec_circuit.clone())?,
+        })
+    }
+    pub fn map_circuit_unwrap<F>(&self, f: F, name: Option<Option<String>>) -> Self
+    where
+        F: Fn(CircuitRc) -> CircuitRc,
+    {
+        ModuleNodeSpec {
+            input_specs: self.input_specs.clone(),
+            name: name.unwrap_or(self.name.clone()),
+            spec_circuit: f(self.spec_circuit.clone()),
+        }
+    }
 }
 #[pymethods]
 impl ModuleNodeSpec {
@@ -179,6 +212,11 @@ impl ModuleNodeSpec {
         }
     }
 
+    #[pyo3(name = "map_circuit")]
+    pub fn map_circuit_py(&self, f: PyObject, name: Option<String>) -> Result<Self> {
+        self.map_circuit(|x| pycall!(f, (x,), anyhow), Some(name))
+    }
+
     #[staticmethod]
     pub fn new_auto(circuit: CircuitRc, name: Option<String>) -> Result<Self> {
         let mut input_specs_dict: HashMap<usize, ModuleNodeArgSpec> = HashMap::default();
@@ -227,10 +265,10 @@ impl ModuleNodeSpec {
         input_specs: Vec<(CircuitRc, ModuleNodeArgSpec)>,
         name: Option<String>,
         require_all_inputs: bool,
-    ) -> Option<Self> {
+    ) -> Result<Self> {
         let mut new_input_specs: Vec<Option<ModuleNodeArgSpec>> = vec![None; input_specs.len()];
         let spec_circuit = deep_map_op_context_preorder_stoppable(
-            circuit,
+            circuit.clone(),
             &|circuit,
               c: &mut (
                 &mut Vec<Option<ModuleNodeArgSpec>>,
@@ -257,9 +295,19 @@ impl ModuleNodeSpec {
             },
             &mut (&mut new_input_specs, &input_specs),
             &mut Default::default(),
-        )?;
+        )
+        .unwrap_or(circuit);
         let new_input_specs: Vec<ModuleNodeArgSpec> = if require_all_inputs {
-            new_input_specs.into_iter().collect::<Option<Vec<_>>>()?
+            let er = new_input_specs
+                .iter()
+                .cloned()
+                .collect::<Option<Vec<_>>>()
+                .ok_or_else(|| ConstructError::ModuleNodeExtractNotPresent {
+                    subcirc: input_specs[new_input_specs.iter().position(|x| x.is_none()).unwrap()]
+                        .0
+                        .clone(),
+                });
+            er?
         } else {
             new_input_specs
                 .into_iter()
@@ -267,7 +315,7 @@ impl ModuleNodeSpec {
                 .collect::<Option<Vec<_>>>()
                 .unwrap()
         };
-        Some(Self {
+        Ok(Self {
             spec_circuit,
             input_specs: new_input_specs,
             name,
@@ -309,6 +357,7 @@ pub struct ModuleNodeArgSpec {
 #[pymethods]
 impl ModuleNodeArgSpec {
     #[new]
+    #[args(batchable = "true", expandable = "true")]
     fn new(symbol: Symbol, batchable: bool, expandable: bool) -> Self {
         Self {
             symbol,
@@ -316,6 +365,18 @@ impl ModuleNodeArgSpec {
             expandable,
         }
     }
+    #[staticmethod]
+    #[args(batchable = "true", expandable = "true")]
+    pub fn just_name(circuit: CircuitRc, batchable: bool, expandable: bool) -> Self {
+        Self {
+            symbol: Symbol::new_with_random_uuid(
+                circuit.info().shape.clone(),
+                circuit.name_cloned(),
+            ),
+            batchable,
+            expandable,
+        }
+    }
 }
 
 #[pyclass(extends=PyCircuitBase)]
@@ -486,7 +547,7 @@ impl ModuleNode {
 
     pub fn expand(&self) -> CircuitRc {
         self.spec
-            .expand(&self.nodes)
+            .expand(&self.nodes, self.name_cloned().map(|x| x + "."))
             .expect("module expansion fail!") // maybe this is supposed to return error instead of panicking?
     }
 }
@@ -500,7 +561,7 @@ pub fn inline_all_modules(circuit: CircuitRc) -> CircuitRc {
 }
 use std::cell::RefCell;
 thread_local! {
-    static MODULE_EXPANSIONS: RefCell<HashMap<(ModuleNodeSpec, Vec<HashBytes>), CircuitRc>> =
+    static MODULE_EXPANSIONS: RefCell<HashMap<(ModuleNodeSpec, Vec<HashBytes>,Option<String>), CircuitRc>> =
         RefCell::new(HashMap::default());
     static MODULE_EXPANSIONS_SHAPE: RefCell<HashMap<(ModuleNodeSpec, Vec<Shape>), CircuitRc>> =
         RefCell::new(HashMap::default());
