@@ -132,6 +132,17 @@ pub fn remove_add_few_input(add: &Add) -> Option<CircuitRc> {
 }
 
 pub fn make_einsum_ints_same_one_layer(einsum: &Einsum) -> Einsum {
+    make_einsum_ints_same_one_layer_and_int_info(einsum).0
+}
+
+pub fn make_einsum_ints_same_one_layer_and_int_info(
+    einsum: &Einsum,
+) -> (
+    Einsum,
+    UnionFind,
+    HashMap<u8, u8>,
+    Vec<Option<HashMap<u8, u8>>>,
+) {
     let mut top_int_map: HashMap<u8, u8> = HashMap::default();
     let mut next_global_int: u8 = 0;
     for i in einsum.axes_in_input() {
@@ -172,7 +183,7 @@ pub fn make_einsum_ints_same_one_layer(einsum: &Einsum) -> Einsum {
         })
         .collect();
 
-    let new_args = zip(&einsum.args, int_maps)
+    let new_args = zip(&einsum.args, int_maps.clone())
         .map(|((node, ints), int_map_op)| {
             let new_operand_axes = ints
                 .iter()
@@ -214,7 +225,7 @@ pub fn make_einsum_ints_same_one_layer(einsum: &Einsum) -> Einsum {
             }
         })
         .collect();
-    einsum.evolve(
+    let ein_out = einsum.evolve(
         Some(new_args),
         Some(
             einsum
@@ -223,7 +234,9 @@ pub fn make_einsum_ints_same_one_layer(einsum: &Einsum) -> Einsum {
                 .map(|x| unionfind.find(top_int_map[x] as usize) as u8)
                 .collect(),
         ),
-    )
+    );
+
+    (ein_out, unionfind, top_int_map, int_maps)
 }
 
 #[pyfunction]
