@@ -252,7 +252,7 @@ pub fn optimize_circuit(circuit: CircuitRc, context: &mut OptimizationContext) -
         circuit.compiler_print();
     }
     if context.settings.save_circ {
-        let s = repr_circuit_deep_compiler(&circuit, true, true);
+        let s = repr_circuit_deep_compiler(&circuit, true, true, false);
         let f_name = format!("cached_circ_{}.circ", circuit.info().hash_usize());
         python_println!("Save circ to {}", f_name);
 
@@ -314,7 +314,10 @@ pub fn optimize_to_schedule(
         python_println!("Optimizing verbose {}", verbose)
     }
     if settings.capture_and_print {
-        python_println!("{}", repr_circuit_deep_compiler(&**circuit, true, false));
+        python_println!(
+            "{}",
+            repr_circuit_deep_compiler(&**circuit, true, false, false)
+        );
     }
     let optimized_circuit = timed!(optimize_circuit(circuit, context), 10, verbose >= 1);
     // return evaluate_fn(&*optimized_circuit).unwrap();
