@@ -2,6 +2,7 @@ use circuit_base::{
     deep_map_op_context, deep_map_preorder_unwrap, deep_map_unwrap, prelude::*,
     visit_circuit_unwrap, Add, Concat, GeneralFunction, Index,
 };
+use itertools::Itertools;
 use num_bigint::BigUint;
 use pyo3::prelude::*;
 use rr_util::{
@@ -379,6 +380,7 @@ pub fn deep_heuristic_nest_adds(circ: CircuitRc) -> CircuitRc {
         }
     });
     // TODO: profile and optimize these
+
     let mut intersections: HashSet<Add> = HashSet::default();
     for circ in &seen_adds {
         for circ2 in &seen_adds {
@@ -401,7 +403,13 @@ pub fn deep_heuristic_nest_adds(circ: CircuitRc) -> CircuitRc {
         for cand_sub in &seen_adds {
             if cand_sub.nodes.len() >= 2 {
                 for cand_sup in &seen_adds {
-                    if cand_sub != cand_sup {
+                    if cand_sub != cand_sup
+                        && cand_sup
+                            .nodes
+                            .iter()
+                            .position(|x| x.info().hash == cand_sub.info().hash)
+                            .is_some()
+                    {
                         if let Some(new) = extract_add(cand_sup, cand_sub) {
                             return Some((cand_sup.clone(), new));
                         }
