@@ -2,13 +2,14 @@ use std::vec;
 
 use anyhow::{bail, Context, Result};
 use circuit_base::{computational_node::EinsumArgs, CircuitNode, CircuitRc, Einsum};
-use circuit_rewrites::algebraic_rewrite::make_einsum_ints_same_one_layer;
+use circuit_rewrites::algebraic_rewrite::make_einsum_ints_same_one_layer_and_int_info;
 use get_update_node::{IterativeMatcher, IterativeMatcherRc, MatcherRc};
 use macro_rules_attribute::apply;
 use pyo3::{exceptions::PyValueError, prelude::*};
 use rr_util::{
     python_error_exception,
     rearrange_spec::{check_permutation, PermError},
+    union_find::UnionFind,
     util::{flip_result_op, is_unique, AxisInt, EinsumAxes},
 };
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
@@ -54,15 +55,18 @@ pub struct NestEinsumsSpecInfo {
     pub spec: NestEinsumsSpecMultiple,
     pub name: Option<String>,
     pub out_axes_perm: Option<EinsumAxes>,
+    pub shrink_out_axes: bool,
 }
 
 #[pymethods]
 impl NestEinsumsSpecInfo {
     #[new]
+    #[args(name = "None", out_axes_perm = "None", shrink_out_axes = "false")]
     pub fn new(
         spec: NestEinsumsSpecMultiple,
         name: Option<String>,
         out_axes_perm: Option<EinsumAxes>,
+        shrink_out_axes: bool,
     ) -> Result<Self> {
         if let Some(axes) = &out_axes_perm {
             check_permutation(axes).context("out_axes_perm not permutation")?
@@ -71,6 +75,7 @@ impl NestEinsumsSpecInfo {
             spec,
             name,
             out_axes_perm,
+            shrink_out_axes,
         })
     }
 }
@@ -93,16 +98,26 @@ mod nest_einsum_match_prelude {
 pub struct NestEinsumsIdxsInfo {
     pub idxs: NestEinsumsIdxsItem,
     pub name: Option<String>,
+    pub out_axes: Option<EinsumAxes>,
     pub out_axes_perm: Option<EinsumAxes>,
+    pub shrink_out_axes: bool,
 }
 
 #[pymethods]
 impl NestEinsumsIdxsInfo {
     #[new]
+    #[args(
+        name = "None",
+        shrink_out_axes = "false",
+        out_axes = "None",
+        out_axes_perm = "None"
+    )]
     fn new(
         idxs: NestEinsumsIdxsItem,
         name: Option<String>,
+        out_axes: Option<EinsumAxes>,
         out_axes_perm: Option<EinsumAxes>,
+        shrink_out_axes: bool,
     ) -> Result<Self> {
         if let Some(axes) = &out_axes_perm {
             check_permutation(axes).context("sorted_out_axes_perm not permutation")?
@@ -111,11 +126,38 @@ impl NestEinsumsIdxsInfo {
         Ok(Self {
             idxs,
             name,
+            out_axes,
             out_axes_perm,
+            shrink_out_axes,
         })
     }
 }
 
+impl NestEinsumsIdxsInfo {
+    pub fn map_out_axes(&self, mapping: impl Fn(u8) -> u8 + Clone) -> Self {
+        let new_idxs = match &self.idxs {
+            NestEinsumsIdxsItem::Single(x) => NestEinsumsIdxsItem::Single(*x),
+            NestEinsumsIdxsItem::Many(items) => NestEinsumsIdxsItem::Many(
+                items
+                    .iter()
+                    .map(|x| x.map_out_axes(mapping.clone()))
+                    .collect(),
+            ),
+        };
+
+        Self {
+            idxs: new_idxs,
+            name: self.name.clone(),
+            shrink_out_axes: self.shrink_out_axes,
+            out_axes: self
+                .out_axes
+                .as_ref()
+                .map(|axes| axes.iter().cloned().map(mapping).collect()),
+            out_axes_perm: self.out_axes_perm.clone(),
+        }
+    }
+}
+
 #[derive(Debug, Clone, FromPyObject)]
 pub enum NestEinsumsIdxsItem {
     Single(usize),
@@ -145,7 +187,7 @@ impl NestEinsumsIdxsItem {
 
     fn insert_exact_subset(
         &self,
-        name_out_axes: Option<Option<String>>,
+        name_out_axes: Option<(Option<String>, Option<EinsumAxes>)>,
         exact_subsets: &mut HashMap<(usize, usize), NestEinsumsIdxsInfo>,
     ) -> Option<(usize, usize)> {
         let out = match self {
@@ -155,10 +197,10 @@ impl NestEinsumsIdxsItem {
                     items
                         .iter()
                         .fold((usize::MAX, usize::MIN), |(start, end), item| {
-                            if let Some((new_start, new_end)) = item
-                                .idxs
-                                .insert_exact_subset(Some(item.name.clone()), exact_subsets)
-                            {
+                            if let Some((new_start, new_end)) = item.idxs.insert_exact_subset(
+                                Some((item.name.clone(), item.out_axes.clone())),
+                                exact_subsets,
+                            ) {
                                 // we assume contiguous, so this is valid!
                                 if end != usize::MIN {
                                     assert_eq!(end, new_start);
@@ -177,10 +219,10 @@ impl NestEinsumsIdxsItem {
                 }
             }
         };
-        if let Some(name) = name_out_axes {
+        if let Some((name, out_axes)) = name_out_axes {
             exact_subsets.insert(
                 out,
-                NestEinsumsIdxsInfo::new(self.clone(), name, None).unwrap(),
+                NestEinsumsIdxsInfo::new(self.clone(), name, out_axes, None, false).unwrap(),
             );
         }
         Some(out)
@@ -355,7 +397,7 @@ impl NestEinsumsSpec {
         &self,
         all_ints: &[usize],
         mapping: &HashMap<(usize, usize), NestEinsumsIdxsInfo>,
-    ) -> (Option<String>, Option<EinsumAxes>) {
+    ) -> (Option<String>, bool, Option<EinsumAxes>, Option<EinsumAxes>) {
         let start_r = *all_ints.iter().min().unwrap();
         let end_r = *all_ints.iter().max().unwrap() + 1;
         let orig_info = if all_ints.len() == end_r - start_r {
@@ -363,14 +405,26 @@ impl NestEinsumsSpec {
         } else {
             None
         };
+        let out_axes = orig_info.map(|x| x.out_axes.clone()).flatten();
         match self {
             Self::Info(NestEinsumsSpecInfo {
                 name,
+                shrink_out_axes,
                 out_axes_perm,
                 ..
-            }) => (name.clone(), out_axes_perm.clone()),
+            }) => (
+                name.clone(),
+                *shrink_out_axes,
+                out_axes,
+                out_axes_perm.clone(),
+            ),
 
-            Self::Sub(_) => (orig_info.map(|x| x.name.clone()).flatten(), None),
+            Self::Sub(_) => (
+                orig_info.map(|x| x.name.clone()).flatten(),
+                false,
+                out_axes,
+                None,
+            ),
         }
     }
 
@@ -406,13 +460,16 @@ impl NestEinsumsSpec {
                     return Ok((vec![], all_ints));
                 }
 
-                let (name, out_axes_perm) = self.get_info(&all_ints, mapping);
+                let (name, shrink_out_axes, out_axes, out_axes_perm) =
+                    self.get_info(&all_ints, mapping);
 
                 (
                     vec![NestEinsumsIdxsInfo::new(
                         NestEinsumsIdxsItem::Many(items_vec.into_iter().flatten().collect()),
                         name,
+                        out_axes,
                         out_axes_perm,
+                        shrink_out_axes,
                     )
                     .unwrap()],
                     all_ints,
@@ -432,11 +489,14 @@ impl NestEinsumsSpec {
                 let out_idxs = if *flat && !is_outer {
                     rest_items
                 } else {
-                    let (name, out_axes_perm) = self.get_info(&all_ints, mapping);
+                    let (name, shrink_out_axes, out_axes, out_axes_perm) =
+                        self.get_info(&all_ints, mapping);
                     vec![NestEinsumsIdxsInfo::new(
                         NestEinsumsIdxsItem::Many(rest_items),
                         name,
+                        out_axes,
                         out_axes_perm,
+                        shrink_out_axes,
                     )
                     .unwrap()]
                 };
@@ -447,11 +507,17 @@ impl NestEinsumsSpec {
     }
 }
 
-pub fn collect_named_idxs(args: &[(CircuitRc, Option<NamedEnumeration>)]) -> NamedEnumeration {
+pub fn collect_named_idxs(
+    args: &[(CircuitRc, Option<NamedEnumeration>)],
+    union_find: &UnionFind,
+    int_maps: &[Option<HashMap<u8, u8>>],
+) -> NamedEnumeration {
+    assert_eq!(args.len(), int_maps.len());
     let mut running_count = 0;
     let out = args
         .into_iter()
-        .map(|(circ, maybe_items)| {
+        .zip(int_maps)
+        .map(|((circ, maybe_items), int_map)| {
             let (new_item, additional_count) = match maybe_items {
                 Some(items) => {
                     // not efficient but whatever
@@ -460,7 +526,23 @@ pub fn collect_named_idxs(args: &[(CircuitRc, Option<NamedEnumeration>)]) -> Nam
                 None => (NestEinsumsIdxsItem::Single(running_count), 1),
             };
             running_count += additional_count;
-            NestEinsumsIdxsInfo::new(new_item, circ.name_cloned(), None).unwrap()
+
+            let mut out = NestEinsumsIdxsInfo::new(
+                new_item,
+                circ.name_cloned(),
+                circ.as_einsum().map(|x| x.out_axes.clone()),
+                None,
+                false,
+            )
+            .unwrap();
+
+            assert_eq!(circ.is_einsum(), int_map.is_some());
+
+            if let Some(int_map) = int_map {
+                let get_new_nums = |x: u8| union_find.find(int_map[&x] as usize) as u8;
+                out = out.map_out_axes(get_new_nums) // quadratic in tree depth fwiw
+            }
+            out
         })
         .collect();
     NamedEnumeration::new(NestEinsumsIdxsItem::Many(out))
@@ -499,7 +581,8 @@ pub fn einsum_flatten_impl(
         .map(|c| einsum_flatten_impl(c, new_matcher.clone()))
         .collect::<Result<Vec<_>>>()?;
     let einsum = einsum.map_children_unwrap_idxs(|i| rec_on_children[i].0.clone());
-    let einsum = make_einsum_ints_same_one_layer(&einsum);
+    let (einsum, unionfind, _, per_arg_int_maps) =
+        make_einsum_ints_same_one_layer_and_int_info(&einsum);
 
     let new_args = einsum
         .args
@@ -517,7 +600,11 @@ pub fn einsum_flatten_impl(
 
     Ok((
         Einsum::nrc(new_args, einsum.out_axes.clone(), einsum.name_cloned()),
-        Some(collect_named_idxs(&rec_on_children)),
+        Some(collect_named_idxs(
+            &rec_on_children,
+            &unionfind,
+            &per_arg_int_maps,
+        )),
     ))
 }
 
@@ -544,14 +631,14 @@ pub fn nest_einsums(
             circuit: einsum.crc()
         })
     }
-    let flat_einsum = flat_einsum.as_einsum().unwrap().normalize_ints();
+    let flat_einsum = flat_einsum.as_einsum().unwrap();
     let enumeration = enumeration.unwrap();
     let new_item = spec.convert_to_named_idxs(
         &enumeration.exact_subset_to_named_idxs(),
         &flat_einsum.input_circuits().collect::<Vec<_>>(),
     )?;
 
-    nest_flat_einsum_strict(&flat_einsum, new_item)
+    Ok(nest_flat_einsum_strict(&flat_einsum, new_item)?.normalize_ints())
 }
 
 #[pyfunction]
@@ -609,9 +696,25 @@ pub fn nest_flat_einsum_strict_rec(
                         all_other_ints.union(&out_axes_set).cloned().collect();
                     let out_ints = all_other_ints.intersection(my_ints).cloned();
 
-                    // get out axes
-                    let mut sub_out_axes: EinsumAxes = out_ints.collect();
-                    sub_out_axes.sort();
+                    let mut sub_out_axes = if let Some(sub_out_axes) = spec.out_axes {
+                        let out_ints_actual = sub_out_axes.iter().cloned().collect::<HashSet<_>>();
+                        assert!(out_ints_actual.is_subset(my_ints));
+                        let minimal_out_num_set: HashSet<u8> = out_ints.collect();
+                        assert!(out_ints_actual.is_superset(&minimal_out_num_set));
+                        if spec.shrink_out_axes {
+                            // keep ordering for user simplicity
+                            sub_out_axes
+                                .into_iter()
+                                .filter(|x| minimal_out_num_set.contains(x))
+                                .collect()
+                        } else {
+                            sub_out_axes
+                        }
+                    } else {
+                        let mut sub_out_axes: EinsumAxes = out_ints.collect();
+                        sub_out_axes.sort();
+                        sub_out_axes
+                    };
 
                     // then permute
                     if let Some(perm) = spec.out_axes_perm {
