@@ -24,9 +24,8 @@ use crate::{
     filter_by_variant,
     lru_cache::TensorCacheRrfs,
     py_types::{Tensor, PY_UTILS},
-    pycall, python_error_exception,
-    rrfs::{save_tensor_rrfs, tensor_from_hash_prefix},
-    sv,
+    pycall, python_error_exception, sv,
+    tensor_db::{get_tensor_prefix, save_tensor},
     util::{cumsum, HashBytes},
 };
 pub type Shape = Sv<[usize; 6]>; // TODO: maybe this should be struct which preserves invariant that dim shapes >= 0?
@@ -677,7 +676,7 @@ impl TensorIndex {
                         + &slice.stop.map(|i| i.to_string()).unwrap_or("".to_owned())
                 }
                 TensorAxisIndex::Tensor(tensor) => {
-                    save_tensor_rrfs((**tensor).clone()).unwrap();
+                    save_tensor((**tensor).clone(), false).unwrap();
                     "t".to_owned() + &base16::encode_lower(&tensor.hash().unwrap())[..10]
                 }
             });
@@ -716,7 +715,7 @@ impl TensorIndex {
                             .as_mut()
                             .map(|tc| tc.get_tensor(no_t.to_owned()))
                             .unwrap_or_else(|| {
-                                tensor_from_hash_prefix(no_t).context("tensor hash from string")
+                                get_tensor_prefix(no_t).context("tensor hash from string")
                             })
                             .and_then(|x| {
                                 Ok(TensorAxisIndex::Tensor(
