@@ -337,8 +337,22 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     )?)?;
 
     m.add_function(wrap_pyfunction!(circuit_base::print_circuit_type_check, m)?)?;
+    m.add_function(wrap_pyfunction!(
+        circuit_rewrites::module_rewrite::extract_rewrite_raw,
+        m
+    )?)?;
     m.add_function(wrap_pyfunction!(rr_util::py_types::hash_tensor, m)?)?;
 
+    m.add_function(wrap_pyfunction!(rr_util::tensor_db::save_tensor, m)?)?;
+    m.add_function(wrap_pyfunction!(rr_util::tensor_db::get_tensor_prefix, m)?)?;
+    m.add_function(wrap_pyfunction!(
+        rr_util::tensor_db::sync_all_unsynced_tensors,
+        m
+    )?)?;
+    m.add_function(wrap_pyfunction!(
+        rr_util::tensor_db::migrate_tensors_from_old_dir,
+        m
+    )?)?;
     m.add_class::<nb_operations::nest_einsums::NestEinsumsRest>()?;
     m.add_class::<nb_operations::nest_einsums::NestEinsumsSpecInfo>()?;
     m.add_class::<nb_operations::nest_einsums::NestEinsumsIdxsInfo>()?;
